from __future__ import unicode_literals
from helium import DataPoint, Timeseries
from operator import attrgetter
from functools import partial


def display_map(func, cls, client, include=None):
    def _sensor(self):
        try:
            sensor_id = self.sensor_id
            return sensor_id.split('-')[0]
        except AttributeError:
            return None

    dict = func(client, include=include)
    dict.update([
        ('sensor', _sensor),
        ('timestamp', attrgetter('timestamp')),
        ('port', attrgetter('port')),
        ('value', attrgetter('value')),
    ])
    return dict

DataPoint.display_map = classmethod(partial(display_map, DataPoint.display_map))
Timeseries                      # Empty reference to make clear we re-export



"""Configuration commands."""
import click
from builtins import filter
from helium_commander import (
    Sensor,
    Element,
    Client,
    Configuration,
    Device,
    DeviceConfiguration,
    JSONParamType,
    ResourceParamType,
    device_mac_option
)

pass_client = click.make_pass_decorator(Client)


@click.group()
def cli():
    """Operations on configurations."""
    pass


@cli.command()
@click.argument('config', required=False)
@pass_client
def list(client, config):
    """List configurations.

    Lists one CONFIGuration or all configurations associated with the
    authorized organization.

    """
    include = [DeviceConfiguration]
    if config:
        configs = [Configuration.lookup(client, config, include=include)]
    else:
        configs = Configuration.all(client, include=include)
    Configuration.display(client, configs, include=include)


@cli.command()
@click.argument('value', type=JSONParamType())
@pass_client
def create(client, value):
    """Create a configuration.

    Create a configuration with a given set of JSON attributes.

    """
    config = Configuration.create(client, attributes=value)
    Configuration.display(client, [config])


@cli.command()
@click.argument('config')
@pass_client
def delete(client, config):
    """Delete a configuration.

    Deletes a given CONFIGuration. This will fail if there are already
    are devices targetted for a given configuration.

    """
    config = Configuration.lookup(client, config)
    config.delete()
    click.echo('Deleted {}'.format(config.id))


_dev_config_includes = [Device, Configuration]

def _dev_configs(client, config):
    include = _dev_config_includes
    dev_configs = DeviceConfiguration.all(client, include=include)
    return filter(lambda dc:
                  dc.configuration(use_included=True) == config,
                  dev_configs)




@cli.command()
@click.argument('config')
@pass_client
def device(client, config):
    """List associated device configurations.

    A configuration can be applied to zero or more devices through
    "device configurations".

    This lists all the device configurations associated with the given
    CONFIGuration.

    Note: The id of the listed device configuration is NOT the id of
    the device the configuration is applied to. Look for the DEVICE
    column to see the actual targeted device ids.

    """
    config = Configuration.lookup(client, config)

    dev_configs = _dev_configs(client, config)
    DeviceConfiguration.display(client, dev_configs,
                                include=_dev_config_includes)


@cli.command()
@click.argument('config')
@click.option('--add', type=ResourceParamType(metavar='DEVICE'),
              help="Add devices to a configuration")
@click.option('--remove', type=ResourceParamType(metavar='DEVICE'),
              help="Remove devices from a configuration")
@click.option('--type',
              type=click.Choice(['sensor', 'element']),
              default='sensor',
              help="The type of device to add or remove (default sensor).")
@device_mac_option
@pass_client
def update(client, config, add, remove, type, mac):
    """Updates the devices for a configuration.

    Applies or removes a configuration from one or more given DEVICES.

    The type of the DEVICES to apply the configuration to defaults to
    sensors but can be overridden for the given list of devices using
    the --type option.

    DEVICES can be specified using a comma separated list of either
    short or long resource ids, or in a file using "@filename". The
    file should contain newline separated short or long resource ids.

    Note: A device can only have at most one loaded and one unloaded
    (or pending) configuration associated with it. When you apply a
    configuraiton to a device it is always in unloaded state. Loading
    a configuration does NOT happen immediately and can take some time
    depending on how often the device communicates with the helium
    system.

    If you apply multiple configurations to a device the last one
    wins.

    Note: When you remove a configuration that was already "loaded"
    for a given device, removing that configuration has NO effect
    since the system has already processed that configuration for the
    device.

    """
    config = Configuration.lookup(client, config)
    clazz = {
        'sensor': Sensor,
        'element': Element
    }
    add_devices = add or []
    remove_devices = remove or []
    all_devices = clazz[type].all(client)

    # Process adds
    devices = [clazz[type].lookup(client, d, mac=mac, resources=all_devices)
               for d in add_devices]
    if devices:
        map(lambda d: DeviceConfiguration.create(client,
                                                 device=d,
                                                 configuration=config),
            devices)

    # Process removes
    devices = [clazz[type].lookup(client, d, mac=mac, resources=all_devices)
               for d in remove_devices]
    if devices:
        dev_configs = _dev_configs(client, config)
        devices = frozenset(devices)
        dev_configs = filter(lambda dc:
                             dc.device(use_included=True) in devices,
                             dev_configs)
        map(lambda dc: dc.delete(), dev_configs)

    # Re-fetch to include full device information
    dev_configs = _dev_configs(client, config)
    DeviceConfiguration.display(client, dev_configs,
                                include=_dev_config_includes)



import click
from helium_commander import Client, Organization
from helium_commander import User, Element, Sensor, Label
from helium_commander.commands import timeseries
from helium_commander.commands import metadata
from collections import OrderedDict

pass_client = click.make_pass_decorator(Client)


@click.group()
def cli():
    """Operations on the authorized organization
    """
    pass


@cli.command()
@pass_client
def list(client):
    """Display basic information of the organization.

    Displays basic attributes of the authorized organization.
    """
    include = [User, Element, Sensor]
    org = Organization.singleton(client, include=include)
    Organization.display(client, [org], include=include)


supported_timezones = [
    'UTC',
    'US/Pacific',
    'US/Eastern',
    'US/Central',
    'US/Mountain',
    'US/Pacific',
    'Europe/London'
]


@cli.command()
@click.option('--name',
              help="the new name for the organization")
@click.option('--timezone', type=click.Choice(supported_timezones),
              help="the timezone for the organization")
@pass_client
def update(client, name, timezone):
    """Update the attributes of the organization.

    Updates the attributes of the currently authorized organization.
    """
    org = Organization.singleton(client)
    attributes = OrderedDict()
    if name:
        attributes['name'] = name
    if timezone:
        attributes['timezone'] = timezone
    org = org.update(attributes=attributes)
    include = [User, Element, Sensor]
    org = Organization.singleton(client, include=include)
    Organization.display(client, [org], include=include)


@cli.command()
@pass_client
def sensor(client):
    """List sensors for the organization.

    Lists all sensors for the authorized organization.
    """
    org = Organization.singleton(client)
    Sensor.display(client, org.sensors())


@cli.command()
@pass_client
def element(client):
    """List elements for the organization.

    Lists all elements for the authorized organization.
    """
    org = Organization.singleton(client)
    Element.display(client, org.elements())


@cli.command()
@pass_client
def user(client):
    """List users for the organization.

    Lists all users for the authorized organization.
    """
    org = Organization.singleton(client)
    User.display(client, org.users())


@cli.command()
@pass_client
def label(client):
    """List labels for the organization.

    Lists all labels for the authorized organization.
    """
    org = Organization.singleton(client)
    Label.display(client, org.labels())


cli.add_command(timeseries.cli(Organization, singleton=True,
                               history=False, writable=False))
cli.add_command(metadata.cli(Organization, singleton=True))



from __future__ import unicode_literals
from helium import Configuration, DeviceConfiguration
from future.utils import iteritems

def display_map(cls, client, include=None):
    def _count_dev_configs(self):
        return len(self.device_configurations(use_included=True))

    def _attributes(self):
        return {k: v for k, v in iteritems(vars(self))
                if not (k.startswith('_') or k == 'id')}

    dict = super(Configuration, cls).display_map(client, include=include)

    if include and DeviceConfiguration in include:
        dict['devices'] = _count_dev_configs

    dict.update([
        ('attributes', _attributes),
    ])
    return dict


Configuration.display_map = classmethod(display_map)



from __future__ import unicode_literals
from helium import Sensor
from operator import attrgetter


def card_type(card_id, default):
    return {
        '2': 'blue',
        '5': 'green',
    }.get(card_id, default)


def display_map(cls, client, include=None):
    def _type(self):
        try:
            card = str(self.meta.card.get('id'))
            return card_type(card, card)
        except AttributeError:
            return None

    def _meta(attr):
        def func(self):
            return getattr(self.meta, attr, None)
        return func

    dict = super(Sensor, cls).display_map(client, include=include)
    dict.update([
        ('mac', _meta('mac')),
        ('type', _type),
        ('created', _meta('created')),
        ('seen', _meta('last_seen')),
        ('name', attrgetter('name'))
    ])
    return dict

Sensor.display_map = classmethod(display_map)
